@@ -218,6 +218,7 @@ check_closed(VALUE db)
 	}
 }
 
+#ifndef RPMDB_OPAQUE
 VALUE
 rpm_db_get_root(VALUE db)
 {
@@ -231,6 +232,7 @@ rpm_db_get_home(VALUE db)
 	check_closed(db);
 	return rb_str_new2(RPM_DB(db)->db_home);
 }
+#endif
 
 VALUE
 rpm_db_is_writable(VALUE db)
@@ -284,7 +286,7 @@ static VALUE
 transaction_commit(VALUE tag, VALUE ts)
 {
 	rpm_transaction_commit(0, NULL, ts);
-	
+
 	/* not reached because rpm_transaction_commit() always call rb_throw() */
 	return Qnil;
 }
@@ -430,6 +432,7 @@ rpm_transaction_upgrade(VALUE trans, VALUE pkg, VALUE key)
 	return Qnil;
 }
 
+#ifdef RPMTS_AVAILABLE
 VALUE
 rpm_transaction_available(VALUE trans, VALUE pkg, VALUE key)
 {
@@ -454,12 +457,16 @@ rpm_transaction_available(VALUE trans, VALUE pkg, VALUE key)
 	rpmtransAvailablePackage(RPM_TRANSACTION(trans), RPM_HEADER(pkg),
 					   RSTRING(key)->ptr);
 #else
-	rpmtsAvailablePackage(RPM_TRANSACTION(trans), RPM_HEADER(pkg),
-					   RSTRING(key)->ptr);
+    rb_raise(rb_eNotImpError, "need rpmtsAvailablePackage");
+    // FIXME: What is the analog for rpmtsAvailablePackage
+    // in newer RPM's ?
+    //rpmtsAvailablePackage(RPM_TRANSACTION(trans), RPM_HEADER(pkg),
+    //                      RSTRING(key)->ptr);
 #endif
 
 	return Qnil;
 }
+#endif /* RPMTS_AVAILABLE */
 
 VALUE
 rpm_transaction_delete(VALUE trans, VALUE pkg)
@@ -468,7 +475,7 @@ rpm_transaction_delete(VALUE trans, VALUE pkg)
 	VALUE mi;
 
 	db = rb_ivar_get(trans, id_db);
-	
+
 	if (TYPE(pkg) == T_STRING)
 		mi = rpm_db_init_iterator(db, INT2NUM(RPMDBI_LABEL), pkg);
 	else if (rb_obj_is_kind_of(pkg, rpm_cPackage) != Qfalse) {
@@ -479,9 +486,8 @@ rpm_transaction_delete(VALUE trans, VALUE pkg)
 			VALUE name = rpm_package_aref(pkg,INT2NUM(RPMDBI_LABEL));
 			mi = rpm_db_init_iterator(db, INT2NUM(RPMDBI_LABEL), name);
 		}
-	} else if ( rb_obj_is_kind_of(pkg, rpm_cDependency) ==Qfalse && 
+	} else if ( rb_obj_is_kind_of(pkg, rpm_cDependency) ==Qfalse &&
                     rb_respond_to(pkg,rb_intern("name")) && rb_respond_to(pkg,rb_intern("version"))){
-		VALUE name = rb_funcall(pkg,rb_intern("name"),0);
 		mi = rpm_db_init_iterator(db, INT2NUM(RPMDBI_LABEL),rb_funcall(pkg,rb_intern("name"),0));
 		rpm_mi_set_iterator_version(mi,rb_funcall(pkg,rb_intern("version"),0));
 	} else
@@ -504,6 +510,7 @@ rpm_transaction_delete(VALUE trans, VALUE pkg)
 
 #if RPM_VERSION(4,1,0) <= RPM_VERSION_CODE
 
+#if 0 /* XXX this shouldn't be needed at all */
 /* from rpm-4.2.1/lib/rpmps.c */
 static int
 sameProblem(const rpmProblem p1, const rpmProblem p2)
@@ -521,9 +528,10 @@ sameProblem(const rpmProblem p1, const rpmProblem p2)
 			return 1;
     if (p1->ulong1 != p2->ulong1)
 		return 1;
-	
+
     return 0;
 }
+#endif
 
 static VALUE
 version_new_from_EVR(const char* evr)
@@ -533,12 +541,12 @@ version_new_from_EVR(const char* evr)
 	char *end = NULL;
 	char *tmp_evr = NULL;
 	VALUE version = Qnil;
-	
+
 	tmp_evr = strdup(evr);
 	if (tmp_evr==NULL) { return Qnil; }
 
 	e = tmp_evr;
-	if ( end=strchr(e, ':') ) {
+	if ( (end=strchr(e, ':')) ) {
 		/* epoch is found */
 		*end = '\0';
 		vr = end+1;
@@ -639,8 +647,64 @@ rpm_transaction_check(VALUE trans)
 
 	rc = rpmtsCheck(RPM_TRANSACTION(trans));
 	ps = rpmtsProblems(RPM_TRANSACTION(trans));
+	/* get rid of duplicate problems */
+	rpmpsTrim(ps, RPMPROB_FILTER_NONE);
 	num = rpmpsNumProblems(ps);
 
+#ifdef RPMPS_OPAQUE
+	rpmpsi psi = rpmpsInitIterator(ps);
+	if (num > 0) {
+		list = rb_ary_new();
+	}
+	while (rpmpsNextIterator(psi) >= 0) {
+		rpmProblem p = rpmpsGetProblem(psi);
+		VALUE dep;
+		switch (rpmProblemGetType(p)) {
+		case RPMPROB_REQUIRES: {
+			char *buf = strdup (rpmProblemGetAltNEVR(p));
+			/* TODO: zaki: NULL check*/
+			char *end;
+
+			char *name = buf+2;
+			char *relation = NULL;
+			char *evr = "";
+			rpmsenseFlags sense_flags = 0;
+
+			end = strchr ( name, ' ');
+			if ( end ) {
+				*end = '\0';
+				relation = end + 1;
+				end = strchr ( relation, ' ');
+				if ( end ) {
+					*end = '\0';
+					evr = end + 1;
+				}
+				for ( ; (*relation) != '\0'; relation++ ) {
+					if ( (*relation) == '=' ) {
+						sense_flags |= RPMSENSE_EQUAL;
+					} else if ( (*relation) == '>' ) {
+						sense_flags |= RPMSENSE_GREATER;
+					} else if ( (*relation) == '<' ) {
+						sense_flags |= RPMSENSE_LESS;
+					}
+				}
+			}
+
+			dep = rpm_require_new(name,
+					  rpm_version_new(evr),
+					  sense_flags,
+					  package_new_from_NEVR(
+						rpmProblemGetPkgNEVR(p)
+					  ));
+			free ( buf );
+			rb_ary_push(list, dep);
+			break;
+		}
+		default:
+			break;
+		}
+	}
+#else
 	if (ps != NULL && 0 < num) {
 		rpmProblem p;
 		int i;
@@ -648,7 +712,6 @@ rpm_transaction_check(VALUE trans)
 
 		for (i = 0; i < num; i++) {
 			const char *altNEVR;
-			int j;
 			VALUE dep;
 
 			p = ps->probs + i;
@@ -657,6 +720,7 @@ rpm_transaction_check(VALUE trans)
 			if (p->ignoreProblem)
 				continue;
 
+#if 0 /* XXX shouldn't be needed at all due to rpmpsTrim() */
 			/* Filter already appended problems. */
 			for (j = 0; j < i; j++) {
 				if (!sameProblem(p, ps->probs + j))
@@ -664,6 +728,7 @@ rpm_transaction_check(VALUE trans)
 			}
 			if (j < i)
 				continue;
+#endif
 
 			if ( p->type == RPMPROB_REQUIRES ) {
 				char *buf = strdup ( altNEVR );
@@ -689,7 +754,7 @@ rpm_transaction_check(VALUE trans)
 							sense_flags |= RPMSENSE_EQUAL;
 						} else if ( (*relation) == '>' ) {
 							sense_flags |= RPMSENSE_GREATER;
-						} else if ( (*relation), '<' ) {
+						} else if ( (*relation) == '<' ) {
 							sense_flags |= RPMSENSE_LESS;
 						}
 					}
@@ -725,8 +790,9 @@ rpm_transaction_check(VALUE trans)
 #endif
         }
 	}
+#endif /* RPMPS_OPAQUE */
 	ps = rpmpsFree(ps);
-		
+
 	return list;
 #endif
 }
@@ -892,15 +958,15 @@ rpm_transaction_commit(int argc, VALUE* argv, VALUE trans)
 
 	if (rb_block_given_p() == Qtrue) {
 		rpmtsSetNotifyCallback(RPM_TRANSACTION(trans),
-							   transaction_callback,(void *)trans);
+							   (rpmCallbackFunction)transaction_callback,(void *)trans);
 	}else{
 		VALUE keys;
-		
+
 		/* rpmcli.h:extern int rpmcliPackagesTotal; */
 		rpmcliPackagesTotal = 0;
-		
+
 		keys = rpm_transaction_keys(trans);
-		
+
 		if (!NIL_P(keys))
 			rpmcliPackagesTotal = NUM2INT(rb_funcall(keys,rb_intern("length"),0));
 
@@ -912,6 +978,20 @@ rpm_transaction_commit(int argc, VALUE* argv, VALUE trans)
 
 	{
 	VALUE list = rb_ary_new();
+#ifdef RPMPS_OPAQUE
+	rpmpsi psi = rpmpsInitIterator(ps);
+	while (rpmpsNextIterator(psi) >= 0) {
+		rpmProblem p = rpmpsGetProblem(psi);
+		VALUE prb = rb_struct_new(rpm_sProblem,
+					INT2NUM(rpmProblemGetType(p)),
+					(VALUE)rpmProblemGetKey(p),
+					package_new_from_NEVR(
+						rpmProblemGetAltNEVR(p)+2
+					),
+					rb_str_new2(rpmProblemString(p)));
+		rb_ary_push(list, prb);
+	}
+#else
 	if (ps != NULL && rpmpsNumProblems(ps) > 0) {
 		register int i;
 
@@ -927,10 +1007,11 @@ rpm_transaction_commit(int argc, VALUE* argv, VALUE trans)
 			rb_ary_push(list, prb);
 		}
 	}
+#endif
 	rb_ivar_set(trans, id_pl, list);
 	}
 	if (ps) ps = rpmpsFree(ps);
-	
+
 #endif
 
 	rb_ivar_set(trans, id_commited, Qtrue);
@@ -967,9 +1048,9 @@ rpm_db_init_iterator(VALUE db, VALUE key, VALUE val)
 	}
 
 	mi = ALLOC_N(rpm_mi_t,1);
-	if (mi->mi = rpmdbInitIterator(RPM_DB(db), NUM2INT(rb_Integer(key)),
+	if ((mi->mi = rpmdbInitIterator(RPM_DB(db), NUM2INT(rb_Integer(key)),
 						   NIL_P(val) ? NULL : RSTRING(val)->ptr,
-						   NIL_P(val) ? 0 : RSTRING(val)->len)){
+                           NIL_P(val) ? 0 : RSTRING(val)->len))){
 		mi->db = (rpm_db_t*)DATA_PTR(db);
 		db_ref(mi->db);
 		return Data_Wrap_Struct(rpm_cMatchIterator, NULL, mi_free, mi);
@@ -1058,8 +1139,10 @@ Init_rpm_DB(void)
 	rb_define_singleton_method(rpm_cDB, "rebuild", db_s_rebuild, -1);
 	rb_define_method(rpm_cDB, "close", rpm_db_close, 0);
 	rb_define_method(rpm_cDB, "closed?", rpm_db_is_closed, 0);
+#ifndef RPMDB_OPAQUE
 	rb_define_method(rpm_cDB, "root", rpm_db_get_root, 0);
 	rb_define_method(rpm_cDB, "home", rpm_db_get_home, 0);
+#endif
 	rb_define_method(rpm_cDB, "writable?", rpm_db_is_writable, 0);
 	rb_define_method(rpm_cDB, "each_match", rpm_db_each_match, 2);
 	rb_define_method(rpm_cDB, "each", rpm_db_each, 0);
@@ -1095,7 +1178,9 @@ Init_rpm_transaction(void)
 	rb_define_method(rpm_cTransaction, "script_file=", rpm_transaction_set_script_file, 1);
 	rb_define_method(rpm_cTransaction, "install", rpm_transaction_install, 2);
 	rb_define_method(rpm_cTransaction, "upgrade", rpm_transaction_upgrade, 2);
+#ifdef RPMTS_AVAILABLE
 	rb_define_method(rpm_cTransaction, "available", rpm_transaction_available, 2);
+#endif
 	rb_define_method(rpm_cTransaction, "delete", rpm_transaction_delete, 1);
 	rb_define_method(rpm_cTransaction, "check", rpm_transaction_check, 0);
 	rb_define_method(rpm_cTransaction, "order", rpm_transaction_order, 0);
