@@ -104,12 +104,29 @@ package_s_create(VALUE klass, VALUE name, VALUE version)
   return pkg;
 }
 
+static rpmRC read_header_from_file(FD_t fd, const char *filename, Header *hdr)
+{
+	rpmRC rc;
+//#if RPM_VERSION_CODE < RPM_VERSION(4,6,0)
+#if 0
+    /* filename is ignored */
+    Header sigs;
+	rc = rpmReadPackageInfo(fd, &sigs, hdr);
+    headerFree(sigs);
+#else
+	rpmts ts = rpmtsCreate();
+	rc = rpmReadPackageFile(ts, fd, filename, hdr);
+	rpmtsFree(ts);
+#endif
+    return rc;
+}
+
 static VALUE
 package_s_open(VALUE klass, VALUE filename)
 {
 	VALUE pkg = Qnil;
 	FD_t fd;
-	Header hdr, sigs;
+	Header hdr;
 	rpmRC rc;
 
 	if (TYPE(filename) != T_STRING) {
@@ -121,7 +138,8 @@ package_s_open(VALUE klass, VALUE filename)
 		rb_raise(rb_eRuntimeError, "can not open file %s",
 				 RSTRING(filename)->ptr);
 	}
-	rc = rpmReadPackageInfo(fd, &sigs, &hdr);
+
+    rc = read_header_from_file(fd, RSTRING(filename)->ptr, &hdr);
 	Fclose(fd);
 
 	switch (rc) {
@@ -133,7 +151,7 @@ package_s_open(VALUE klass, VALUE filename)
 		/* MO_TODO: zaki: should I warn these two status to users? */
 #endif
 	case RPMRC_OK:
-		headerFree(sigs);
+		//headerFree(sigs);
 		pkg = package_new_from_header(klass, hdr);
 		headerFree(hdr);
 		break;
@@ -175,7 +193,7 @@ package_s_load(VALUE klass, VALUE str)
 	Fseek(fd, 0, SEEK_SET);
 	hdr = headerRead(fd, HEADER_MAGIC_YES);
 	Fclose(fd);
-	Unlink(RSTRING(temp)->ptr);
+	unlink(RSTRING(temp)->ptr);
 
 	if (!hdr) {
 		rb_raise(rb_eArgError, "unable load RPM::Package");
@@ -190,15 +208,18 @@ package_s_load(VALUE klass, VALUE str)
 VALUE
 rpm_package_copy_tags(VALUE from,VALUE to,VALUE tags)
 {
-	int_32 *copy_tags;
+#if RPM_VERSION_CODE < RPM_VERSION(4,6,0)
+#define rpmTag int_32
+#endif
+	rpmTag *copy_tags;
 	int length = NUM2INT(rb_funcall(tags,rb_intern("length"),0));
 	int i;
 
-	copy_tags = ALLOCA_N(int_32,length);
+	copy_tags = ALLOCA_N(rpmTag,length);
 	for (i=0;i<length;i++)
 		copy_tags[i] = NUM2INT(rb_ary_entry(tags, i));
 	headerCopyTags(RPM_HEADER(from),RPM_HEADER(to),copy_tags);
-  return Qnil;
+    return Qnil;
 }
 
 VALUE
@@ -302,6 +323,7 @@ rpm_package_delete_tag(VALUE pkg, VALUE tag)
 	return val;
 }
 
+#if RPM_VERSION_CODE < RPM_VERSION(4,6,0)
 VALUE
 rpm_package_aref(VALUE pkg, VALUE tag)
 {
@@ -311,7 +333,6 @@ rpm_package_aref(VALUE pkg, VALUE tag)
 	rpmTagType type;
 	int_32 count;
 	register int i;
-	register const char* p;
 	int ary_p = 0;
 	int i18n_p = 0;
 
@@ -440,6 +461,141 @@ rpm_package_aref(VALUE pkg, VALUE tag)
  leave:
 	return val;
 }
+#else
+VALUE
+rpm_package_aref(VALUE pkg, VALUE tag)
+{
+	rpmTag tagval = NUM2INT(tag);
+	VALUE val = Qnil;
+    rpmtd tagc = rpmtdNew();
+	int ary_p = 0;
+	int i18n_p = 0;
+
+	if (!headerGet(RPM_HEADER(pkg), tagval, tagc, HEADERGET_MINMEM)) {
+        rpmtdFree(tagc);
+		goto leave;
+	}
+	switch (tagval) {
+	case RPMTAG_DIRINDEXES:
+	case RPMTAG_FILESIZES:
+	case RPMTAG_FILESTATES:
+	case RPMTAG_FILEMODES:
+	case RPMTAG_FILERDEVS:
+	case RPMTAG_FILEMTIMES:
+	case RPMTAG_FILEMD5S:
+	case RPMTAG_FILEFLAGS:
+	case RPMTAG_FILEUSERNAME:
+	case RPMTAG_FILEGROUPNAME:
+	case RPMTAG_PROVIDEFLAGS:
+	case RPMTAG_REQUIREFLAGS:
+	case RPMTAG_CONFLICTFLAGS:
+	case RPMTAG_OBSOLETEFLAGS:
+		ary_p = 1;
+		break;
+
+	case RPMTAG_GROUP:
+	case RPMTAG_SUMMARY:
+	case RPMTAG_DISTRIBUTION:
+	case RPMTAG_VENDOR:
+	case RPMTAG_LICENSE:
+	case RPMTAG_PACKAGER:
+	case RPMTAG_DESCRIPTION:
+		i18n_p = 1;
+		break;
+    default:
+        break;
+	}
+
+	switch (rpmtdType(tagc)) {
+	case RPM_BIN_TYPE:
+		val = rb_str_new2(rpmtdGetString(tagc));
+		break;
+
+	case RPM_CHAR_TYPE:
+	case RPM_INT8_TYPE:
+		if (rpmtdCount(tagc) == 1 && !ary_p) {
+			val = INT2NUM(*rpmtdGetChar(tagc));
+		} else {
+			val = rb_ary_new();
+			rpmtdInit(tagc);
+			while ( rpmtdNext(tagc) != -1 ) {
+				rb_ary_push(val, INT2NUM(*rpmtdGetChar(tagc)));
+			}
+		}
+		break;
+
+	case RPM_INT16_TYPE:
+		if (rpmtdCount(tagc) == 1 && !ary_p) {
+			val = INT2NUM(*rpmtdGetUint16(tagc));
+		} else {
+			val = rb_ary_new();
+			rpmtdInit(tagc);
+			while ( rpmtdNext(tagc) != -1 ) {
+				rb_ary_push(val, INT2NUM(*rpmtdGetUint16(tagc)));
+			}
+		}
+		break;
+
+	case RPM_INT32_TYPE:
+		if (rpmtdCount(tagc) == 1 && !ary_p) {
+			val = INT2NUM(*rpmtdGetUint32(tagc));
+		} else {
+			val = rb_ary_new();
+			rpmtdInit(tagc);
+			while ( rpmtdNext(tagc) != -1) {
+				rb_ary_push(val, INT2NUM(*rpmtdGetUint32(tagc)));
+			}
+		}
+		break;
+
+	case RPM_STRING_TYPE:
+		if (rpmtdCount(tagc) == 1 && !ary_p) {
+			val = rb_str_new2(rpmtdGetString(tagc));
+		} else {
+			val = rb_ary_new();
+			while ( rpmtdNext(tagc) != -1) {
+				rb_ary_push(val, rb_str_new2(rpmtdGetString(tagc)));
+			}
+		}
+		rpmtdFree(tagc);
+		break;
+
+	case RPM_STRING_ARRAY_TYPE:
+		{
+			if (i18n_p) {
+                rpmtd i18ntd = rpmtdNew();
+
+				if (!headerGet(RPM_HEADER(pkg), HEADER_I18NTABLE, i18ntd, HEADERGET_MINMEM)) {
+					goto strary;
+				}
+
+				val = rb_hash_new();
+				rpmtdInit(tagc);
+				while ( rpmtdNext(tagc) != -1) {
+					VALUE lang = rb_str_new2(rpmtdNextString(i18ntd));
+					VALUE str = rb_str_new2(rpmtdGetString(tagc));
+					rb_hash_aset(val, lang, str);
+				}
+                rpmtdFree(i18ntd);
+			} else {
+			strary:
+				val = rb_ary_new();
+				rpmtdInit(tagc);
+				while ( rpmtdNext(tagc) != -1) {
+					rb_ary_push(val, rb_str_new2(rpmtdGetString(tagc)));
+				}
+			}
+            rpmtdFree(tagc);
+		}
+		break;
+
+	default:
+		goto leave;
+	}
+ leave:
+	return val;
+}
+#endif
 
 VALUE
 rpm_package_sprintf(VALUE pkg, VALUE fmt)
@@ -579,6 +735,7 @@ rpm_package_get_files(VALUE pkg)
 	return files;
 }
 
+#if RPM_VERSION_CODE < RPM_VERSION(4,6,0)
 VALUE
 rpm_package_get_dependency(VALUE pkg,int nametag,int versiontag,int flagtag,VALUE (*dependency_new)(const char*,VALUE,int,VALUE))
 {
@@ -617,6 +774,42 @@ rpm_package_get_dependency(VALUE pkg,int nametag,int versiontag,int flagtag,VALU
 	release_entry(flagtype, (void*)flags);
 	return deps;
 }
+#else
+VALUE
+rpm_package_get_dependency(VALUE pkg,int nametag,int versiontag,int flagtag,VALUE (*dependency_new)(const char*,VALUE,int,VALUE))
+{
+	VALUE deps;
+
+    rpmtd nametd = rpmtdNew();
+    rpmtd versiontd = rpmtdNew();
+    rpmtd flagtd = rpmtdNew();
+
+	deps = rb_ary_new();
+
+	if (!headerGet(RPM_HEADER(pkg), nametag, nametd, HEADERGET_MINMEM)) {
+		goto leave;
+	}
+	if (!headerGet(RPM_HEADER(pkg), versiontag, versiontd, HEADERGET_MINMEM)) {
+		goto leave;
+		return deps;
+	}
+	if (!headerGet(RPM_HEADER(pkg), flagtag, flagtd, HEADERGET_MINMEM)) {
+		goto leave;
+		return deps;
+	}
+
+    rpmtdInit(nametd);
+    while ( rpmtdNext(nametd) != -1 ) {
+        rb_ary_push(deps,dependency_new(rpmtdGetString(nametd),rpm_version_new(rpmtdNextString(versiontd)),*rpmtdNextUint32(flagtd),pkg));
+    }
+    
+ leave:
+    rpmtdFree(nametd);
+    rpmtdFree(versiontd);
+    rpmtdFree(flagtd);
+	return deps;
+}
+#endif
 
 VALUE
 rpm_package_get_provides(VALUE pkg)
@@ -642,6 +835,7 @@ rpm_package_get_obsoletes(VALUE pkg)
 	return rpm_package_get_dependency(pkg,RPMTAG_OBSOLETENAME,RPMTAG_OBSOLETEVERSION,RPMTAG_OBSOLETEFLAGS,rpm_obsolete_new);
 }
 
+#if RPM_VERSION_CODE < RPM_VERSION(4,6,0)
 VALUE
 rpm_package_get_changelog(VALUE pkg)
 {
@@ -680,6 +874,47 @@ rpm_package_get_changelog(VALUE pkg)
 	}
 	return cl;
 }
+#else
+VALUE
+rpm_package_get_changelog(VALUE pkg)
+{
+	VALUE cl;
+	rpmtd timetd = rpmtdNew();
+	rpmtd nametd = rpmtdNew();
+	rpmtd texttd = rpmtdNew();
+
+	cl = rb_ary_new();
+
+	if (!headerGet(RPM_HEADER(pkg), RPMTAG_CHANGELOGTIME, timetd, HEADERGET_MINMEM)) {
+		goto leave;
+	}
+	if (!headerGet(RPM_HEADER(pkg), RPMTAG_CHANGELOGNAME, nametd, HEADERGET_MINMEM)) {
+		goto leave;
+	}
+	if (!headerGet(RPM_HEADER(pkg), RPMTAG_CHANGELOGTEXT, texttd, HEADERGET_MINMEM)) {
+		goto leave;
+	}
+
+	rpmtdInit(timetd);
+	rpmtdInit(nametd);
+	rpmtdInit(texttd);
+
+	while ( rpmtdNext(timetd) != -1 ) {
+		VALUE chglog = rb_struct_new(
+			rpm_sChangeLog,
+			rb_time_new((time_t) rpmtdGetUint32(timetd), (time_t)0),
+			rb_str_new2(rpmtdNextString(nametd)),
+			rb_str_new2(rpmtdNextString(texttd)));
+		 rb_ary_push(cl, chglog);
+	}
+
+ leave:
+	rpmtdFree(timetd);
+	rpmtdFree(nametd);
+	rpmtdFree(texttd);
+	return cl;
+}
+#endif
 
 VALUE
 rpm_package_dump(VALUE pkg)
@@ -699,7 +934,7 @@ rpm_package_dump(VALUE pkg)
 	munmap(buf, size);
 
 	Fclose(fd);
-	Unlink(RSTRING(temp)->ptr);
+	unlink(RSTRING(temp)->ptr);
 
 	return dump;
 }
